using Microsoft.AspNetCore.Mvc;
using System.Threading.Tasks;

namespace Ewide.Core.Service
{
    public interface ISysDictDataService
    {
        Task AddDictData(AddDictDataInput input);
        Task ChangeDictDataStatus(UpdateDictDataInput input);
        Task DeleteByTypeId(string dictTypeId);
        Task DeleteDictData(DeleteDictDataInput input);
        Task<dynamic> GetDictData([FromQuery] QueryDictDataInput input);
        Task<dynamic> GetDictDataList([FromQuery] QueryDictDataListInput input);
        Task<dynamic> GetDictDataListByDictTypeId(string dictTypeId);
        Task<dynamic> GetDictDataListByDictTypeIds(SysDictType[] dictTypes);
        Task<dynamic> QueryDictDataPageList([FromQuery] DictDataInput input);
        Task DeleteDictDataBatch(string[] Ids);
        Task UpdateDictData(UpdateDictDataInput input);
    }
}